import get from 'lodash.get';
import cloneDeep from 'lodash.clonedeep';
import { subYears, addYears, parse, isValid, isBefore, isAfter, isEqual } from 'date-fns';
import { buildYup } from 'json-schema-to-yup';
import isArray from 'lodash.isarray';
import { __decorate } from 'tslib';
import 'reflect-metadata';
import { registerDecorator, ValidatorConstraint, IsNumber, IsString, IsNotEmpty, IsArray, IsOptional, IsObject, ValidateNested, IsEnum, Matches, MaxLength, IsBoolean, IsDateString, IsISO8601, IsIn, validate } from 'class-validator';
import { Type, plainToClass } from 'class-transformer';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}
function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;
  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }
  return target;
}

// Desc: Configs for the translation that is used in the validation messages
const translatesConfig = {
  nullable: 'generic-form:validation.nullable',
  required: 'generic-form:validation.required',
  min: 'generic-form:validation.min',
  max: 'generic-form:validation.max',
  maxLength: 'generic-form:validation.max_length',
  minLength: 'generic-form:validation.min_length',
  pattern: 'generic-form:validation.pattern',
  format: 'generic-form:validation.format',
  email: 'generic-form:validation.email',
  minDate: 'generic-form:validation.min_date',
  maxDate: 'generic-form:validation.max_date',
  wrongRange: 'generic-form:validation.wrong_range',
  wrongDate: 'generic-form:validation.wrong_range',
  trim: 'generic-form:validation.trim',
  oneOf: 'generic-form:validation.one_of'
};
// Desc: Enum for the validation error types
var EValidationTypes;
(function (EValidationTypes) {
  EValidationTypes["nullable"] = "nullable";
  EValidationTypes["required"] = "required";
  EValidationTypes["min"] = "min";
  EValidationTypes["max"] = "max";
  EValidationTypes["oneOf"] = "oneOf";
  EValidationTypes["maxLength"] = "maxLength";
  EValidationTypes["minLength"] = "minLength";
  EValidationTypes["pattern"] = "pattern";
  EValidationTypes["format"] = "format";
  EValidationTypes["minDate"] = "minDate";
  EValidationTypes["maxDate"] = "maxDate";
  EValidationTypes["wrongRange"] = "wrongRange";
  EValidationTypes["wrongDate"] = "wrongDate";
  EValidationTypes["trim"] = "trim";
})(EValidationTypes || (EValidationTypes = {}));

var EFieldTypes;
(function (EFieldTypes) {
  EFieldTypes["TEXT"] = "text";
  EFieldTypes["NUMBER"] = "number";
  EFieldTypes["SWITCH"] = "switch";
  EFieldTypes["DATE"] = "date";
  EFieldTypes["DATE_RANGE"] = "dateRange";
  EFieldTypes["AUTOCOMPLETE"] = "autoComplete";
  EFieldTypes["FILE"] = "file";
  EFieldTypes["ARRAY"] = "array";
  EFieldTypes["MOBILE_PHONE"] = "mobilePhone";
  EFieldTypes["MULTIPLE_AUTOCOMPLETE"] = "autoCompleteMultiple";
  EFieldTypes["RADIOBUTTON"] = "radioButton";
  EFieldTypes["YES_NO_RADIOBUTTON"] = "yesNoRadioButton";
  EFieldTypes["EMPTY"] = "empty";
  EFieldTypes["CHECKBOX"] = "checkbox";
  EFieldTypes["RADIOBUTTON_MATRIX"] = "radioButtonMatrix";
  EFieldTypes["RATING"] = "rating";
})(EFieldTypes || (EFieldTypes = {}));
var ESelectTypes;
(function (ESelectTypes) {
  ESelectTypes["COUNTRIES"] = "countries";
  ESelectTypes["RELATIONSHIP"] = "relationship";
})(ESelectTypes || (ESelectTypes = {}));

/**
 * @param formDataChildren - array of form fields
 * @param formSettings
 * @param acc - accumulator, used for recursion
 * @returns {[key: string]: any} - object with the initial values for the form
 * @description getInitialValues function returns an object with the initial values for the form
 * based on the initial values provided in the json schema
 */
const getInitialValues = (formDataChildren, formSettings, acc = {}) => {
  return formDataChildren.reduce((acc, item) => {
    if (isFormGroup(item)) {
      return getInitialValues(item.children, formSettings, acc);
    }
    const overridesObject = get(formSettings, `${String(item.fieldName)}.overrides`, {});
    const initialValue = Object.keys(overridesObject).includes('initialValue') ? overridesObject.initialValue : item.initialValue;
    acc[item.fieldName] = initialValue;
    return acc;
  }, acc);
};
/**
 * @param schema - yup schema
 * @param fieldName - field name to get the rule value from
 * @param ruleName - rule name to get the value from
 * @returns {any} - rule value
 * @description getRuleValueFromSchema function returns the value of a rule from a yup schema
 * based on the field name and rule name. This value is needed for passing meta to translations.
 * @example getRuleValueFromSchema(schema, 'email', 'minLength') => 6
 */
const getRuleValueFromSchema = (schema, fieldName, ruleName) => {
  const describedSchema = schema.describe();
  const rules = get(describedSchema, `fields.${fieldName}.tests`, []);
  const rule = rules.find(rule => rule.name === ruleName);
  return get(rule, `params.${ruleName}`, undefined);
};
/**
 * @param type - validation type
 * @param value - value to be validated
 * @param t - translation function(optional)
 * @param validationParam - validation param(meta) to be passed to the translation function(optional) or error object
 * @returns {any} - error object or function, or translated string
 * @description getFieldError function returns an error object or function, or translated string based on the validation type
 */
const getFieldError = (type, value, t, validationParam) => {
  let translationKey = get(translatesConfig, `${type}`);
  switch (type) {
    case EValidationTypes.nullable:
      return t ? t(translationKey) : createErrorObject(translationKey);
    case EValidationTypes.required:
      return t ? t(translationKey) : createErrorObject(translationKey);
    case EValidationTypes.min:
      return t ? ({
        min
      }) => t(translationKey, {
        param: min
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.max:
      return t ? ({
        max
      }) => t(translationKey, {
        param: max
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.maxLength:
      return t ? ({
        maxLength
      }) => t(translationKey, {
        param: maxLength
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.minLength:
      return t ? ({
        minLength
      }) => t(translationKey, {
        param: minLength
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.format:
      if (value) {
        translationKey = get(translatesConfig, `${type}`) ? get(translatesConfig, `${type}`) : translationKey;
      }
      return t ? t(translationKey) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.trim:
    case EValidationTypes.oneOf:
    case EValidationTypes.pattern:
      return t ? t(translationKey) : createErrorObject(translationKey, {
        param: validationParam
      });
    default:
      return null;
  }
};
/**
 * @param validationObject
 * @param formValues - form values to be validated
 * @returns {IValidationErrors} - object with the validation errors
 * @description makeCrossFieldValidation function makes custom cross-field validation
 * and returns an object with the validation errors
 */
const makeCrossFieldValidation = (validationObject, formValues) => {
  const errors = {};
  try {
    const keys = Object.keys(validationObject);
    keys.forEach(fieldName => {
      const rules = validationObject[fieldName];
      const fieldErrors = {
        property: fieldName,
        constraints: {}
      };
      rules == null || rules.forEach((rule, id) => {
        const {
          errorKey,
          testCondition,
          meta
        } = rule;
        const testFunction = functionFromString(testCondition);
        let shouldIncludeError = false;
        try {
          shouldIncludeError = testFunction(formValues);
        } catch (e) {
          console.log('Error during evaluating custom validation rule. This issue wont cause an error, current validation was skipped', e);
        }
        if (shouldIncludeError) {
          fieldErrors.constraints[id] = createErrorObject(errorKey, meta);
        }
      });
      if (Object.keys(fieldErrors.constraints).length > 0) {
        errors[fieldName] = fieldErrors;
      }
    });
    return errors;
  } catch (e) {
    console.warn('Error in makeCrossFieldValidation: ', e);
    return {};
  }
};
/**
 * @param fieldName - field name
 * @param values - form values
 * @param errors - object with the validation errors
 * @returns {IValidationErrors} - object with the validation errors
 * @description validateDateRangeField function validates date range field
 * and returns an object with the validation errors
 */
const validateDateRangeField = (fieldName, values, errors = {}) => {
  const minDate = subYears(new Date(), 100);
  const maxDate = addYears(new Date(), 100);
  const error = {
    property: fieldName,
    constraints: {}
  };
  const dateRangeUnmapped = get(values, fieldName, [null, null]);
  if (!isArray(dateRangeUnmapped)) {
    error.constraints[EValidationTypes.wrongDate] = {
      translationKey: translatesConfig[EValidationTypes.wrongDate],
      fallbackLanguageTranslation: 'This field expected to be array of strings',
      meta: {}
    };
    if (!errors[fieldName]) {
      errors[fieldName] = error;
    } else {
      errors[fieldName].constraints = _extends({}, errors[fieldName].constraints, error.constraints);
    }
    return errors;
  }
  if (dateRangeUnmapped[0] === null && dateRangeUnmapped[1] === null) {
    return errors;
  }
  const dateRange = dateRangeUnmapped.map(value => value ? parse(value, 'yyyy-MM-dd', new Date()) : null);
  if (!isValid(dateRange[0]) || !isValid(dateRange[1])) {
    error.constraints[EValidationTypes.wrongDate] = {
      translationKey: translatesConfig[EValidationTypes.wrongDate],
      fallbackLanguageTranslation: 'Some date in range is invalid',
      meta: {}
    };
  }
  if (!dateRange[0] || !dateRange[1]) {
    error.constraints[EValidationTypes.required] = {
      translationKey: translatesConfig[EValidationTypes.required],
      fallbackLanguageTranslation: 'Some date in range is empty',
      meta: {}
    };
  }
  if (dateRange[0] && isBefore(dateRange[0], minDate) || dateRange[1] && isBefore(dateRange[1], minDate)) {
    error.constraints[EValidationTypes.minDate] = {
      translationKey: translatesConfig[EValidationTypes.minDate],
      fallbackLanguageTranslation: 'Date range should be between 100 years ago and 100 years in the future',
      meta: {}
    };
  }
  if (dateRange[1] && isAfter(dateRange[1], maxDate) || dateRange[0] && isAfter(dateRange[0], maxDate)) {
    error.constraints[EValidationTypes.maxDate] = {
      translationKey: translatesConfig[EValidationTypes.maxDate],
      fallbackLanguageTranslation: 'Date range should be between 100 years ago and 100 years in the future',
      meta: {}
    };
  }
  if (dateRange[0] && dateRange[1] && (isAfter(dateRange[0], dateRange[1]) || isEqual(dateRange[0], dateRange[1]))) {
    error.constraints[EValidationTypes.wrongRange] = {
      translationKey: translatesConfig[EValidationTypes.wrongRange],
      fallbackLanguageTranslation: 'Start date should be before end date',
      meta: {}
    };
  }
  if (Object.keys(error.constraints).length > 0) {
    if (!errors[fieldName]) {
      errors[fieldName] = error;
    } else {
      errors[fieldName].constraints = _extends({}, errors[fieldName].constraints, error.constraints);
    }
  }
  return errors;
};
/**
 * @param jsonSchema - json validation schema
 * @param t - translation function(optional)
 * @returns {AnyObjectSchema} - yup validation schema
 * @description buildYupSchema function builds a yup validation schema based on the json validation schema
 */
const buildYupSchema = (jsonSchema, t) => {
  return buildYup(cloneDeep(jsonSchema), t ? {
    errMessages: getErrorMessages(jsonSchema, t)
  } : undefined);
};
/**
 * @param form {IForm} - form json schema
 * @param valuesToValidate {any} - form values to be validated
 * @param skipCrossField {boolean} - skip cross-field validation(optional)
 * @returns {Promise<unknown>} - object with the validation errors or error that has occurred during the validation
 * @description validateValues function validates the form values based on the json form object
 */
const validateValues = async ({
  children,
  jsonSchema,
  formSettings
}, valuesToValidate, skipCrossField = false) => {
  let errors = {};
  const flattenedValues = flattenValues(cloneDeep(valuesToValidate));
  const trimmedValues = trimValuesInObject(flattenedValues);
  try {
    const validationSchema = buildYupSchema(jsonSchema);
    const dateRangeFields = getFieldNamesByType(children, EFieldTypes.DATE_RANGE);
    if (!skipCrossField) {
      const formFieldNames = Object.keys(formSettings);
      const crossFieldValidations = formFieldNames.reduce((acc, el) => {
        if (!formSettings[el].crossFieldValidations) {
          return acc;
        }
        acc[el] = formSettings[el].crossFieldValidations;
        return acc;
      }, {});
      errors = makeCrossFieldValidation(crossFieldValidations, trimmedValues);
    }
    if (dateRangeFields.length > 0) {
      dateRangeFields.forEach(fieldName => {
        errors = validateDateRangeField(fieldName, trimmedValues, errors);
      });
    }
    await validationSchema.validate(trimmedValues, {
      abortEarly: false
    });
  } catch (e) {
    if (get(e, 'name') === 'ValidationError') {
      errors = formatYupValidationErrors(e, errors);
    } else {
      throw e;
    }
  }
  return {
    errors: Object.values(errors)
  };
};

const _excluded = ["grid", "renderCondition", "disabledCondition"],
  _excluded2 = ["crossFieldValidations"];
const buildOptionsFromChildren = (children, formOptions = {}) => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return buildOptionsFromChildren(child.children, acc);
    }
    const fieldLayout = child.grid || {
      md: 12,
      sm: 12,
      xs: 12
    };
    const crossFieldConditions = {};
    if (child.renderCondition) {
      crossFieldConditions.renderCondition = child.renderCondition;
    }
    if (child.disabledCondition) {
      crossFieldConditions.disabledCondition = child.disabledCondition;
    }
    const setting = {
      fieldLayout
    };
    if (crossFieldConditions != null && crossFieldConditions.disabledCondition || crossFieldConditions != null && crossFieldConditions.renderCondition) {
      setting.crossFieldConditions = crossFieldConditions;
    }
    acc[child.fieldName] = setting;
    return acc;
  }, formOptions);
};
const sanitizeChildrenFunction = children => {
  return children.map(child => {
    if (isFormGroup(child)) {
      return _extends({}, child, {
        children: sanitizeChildrenFunction(child.children)
      });
    }
    const sanitizedChild = _objectWithoutPropertiesLoose(child, _excluded);
    return sanitizedChild;
  });
};
const parseFormV1 = form => {
  const {
      submitButtonText,
      cancelButtonText,
      validationObject: {
        crossFieldValidations
      },
      children
    } = form,
    sanitizedValidationObject = _objectWithoutPropertiesLoose(form.validationObject, _excluded2);
  const formOptions = buildOptionsFromChildren(children);
  const fieldsWithCrossFieldValidations = Object.keys(crossFieldValidations);
  fieldsWithCrossFieldValidations.forEach(key => {
    formOptions[key].crossFieldValidations = crossFieldValidations[key];
  });
  const sanitizedChildren = sanitizeChildrenFunction(children);
  const parsedForm = {
    formatVersion: 'parsed',
    jsonSchema: sanitizedValidationObject,
    formSettings: formOptions,
    children: sanitizedChildren,
    submitButtonText
  };
  if (cancelButtonText) {
    parsedForm.cancelButtonText = cancelButtonText;
  }
  return parsedForm;
};

const parseFormV2 = form => {
  const {
    submitButtonText,
    cancelButtonText,
    validationObject,
    crossFieldValidations,
    crossFieldConditions,
    formLayout,
    overrides,
    children
  } = form;
  const allFieldNames = Object.keys(validationObject.properties);
  const formOptions = allFieldNames.reduce((acc, fieldName) => {
    const fieldLayout = formLayout[fieldName] || {
      md: 12,
      sm: 12,
      xs: 12
    };
    const formSetting = {
      fieldLayout
    };
    if (crossFieldValidations != null && crossFieldValidations[fieldName]) {
      formSetting.crossFieldValidations = crossFieldValidations[fieldName];
    }
    if (crossFieldConditions != null && crossFieldConditions[fieldName]) {
      formSetting.crossFieldConditions = crossFieldConditions == null ? void 0 : crossFieldConditions[fieldName];
    }
    if (overrides != null && overrides[fieldName]) {
      formSetting.overrides = overrides[fieldName];
    }
    acc[fieldName] = formSetting;
    return acc;
  }, {});
  const parsedForm = {
    formatVersion: 'parsed',
    jsonSchema: validationObject,
    formSettings: formOptions,
    children,
    submitButtonText
  };
  if (cancelButtonText) {
    parsedForm.cancelButtonText = cancelButtonText;
  }
  return parsedForm;
};
const parseToFormV2 = form => {
  const {
    submitButtonText,
    cancelButtonText,
    formSettings,
    jsonSchema,
    children
  } = form;
  const allFieldNames = Object.keys(formSettings);
  const crossFieldValidations = {};
  const crossFieldConditions = {};
  const formLayout = {};
  const overrides = {};
  allFieldNames.forEach(key => {
    var _formSettings$key, _formSettings$key2, _formSettings$key3, _formSettings$key4;
    const keyCrossFieldConditions = (_formSettings$key = formSettings[key]) == null ? void 0 : _formSettings$key.crossFieldConditions;
    const keyCrossFieldValidations = (_formSettings$key2 = formSettings[key]) == null ? void 0 : _formSettings$key2.crossFieldValidations;
    const keyFieldOverrides = (_formSettings$key3 = formSettings[key]) == null ? void 0 : _formSettings$key3.overrides;
    if (keyCrossFieldConditions) {
      crossFieldConditions[key] = keyCrossFieldConditions;
    }
    if (keyCrossFieldValidations) {
      crossFieldValidations[key] = keyCrossFieldValidations;
    }
    if (keyFieldOverrides) {
      overrides[key] = keyFieldOverrides;
    }
    formLayout[key] = ((_formSettings$key4 = formSettings[key]) == null ? void 0 : _formSettings$key4.fieldLayout) || {
      md: 12,
      sm: 12,
      xs: 12
    };
  });
  const formV2 = {
    formatVersion: 2,
    crossFieldConditions,
    crossFieldValidations,
    formLayout,
    overrides,
    validationObject: jsonSchema,
    children,
    submitButtonText
  };
  if (cancelButtonText) {
    formV2.cancelButtonText = cancelButtonText;
  }
  return formV2;
};

/**
 * @param jsonSchema - jsonSchema validation object
 * @param t - i18n translate function
 * @returns {any}
 * @description getErrorMessages function returns an object with the error messages for each field in the form based on the jsonSchema
 */
const getErrorMessages = (jsonSchema, t) => {
  const errMessages = {};
  Object.keys(jsonSchema.properties).forEach(property => {
    errMessages[property] = {};
    Object.keys(jsonSchema.properties[property]).forEach(key => {
      const value = jsonSchema.properties[property][key];
      errMessages[property][key] = getFieldError(key, value, t);
      if (Object.keys(jsonSchema).includes(key)) {
        errMessages[property].required = getFieldError('required', null, t);
      }
    });
  });
  return errMessages;
};
/**
 * @param translateKey
 * @param param
 * @returns {IValidationError}
 * @description createErrorObject function returns an object with the error message
 * for a field in the form in a proper format
 * @example createErrorObject('required', { param: 'email' }) => { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: { param: 'email' } }
 */
const createErrorObject = (translateKey, param) => ({
  translationKey: translateKey,
  fallbackLanguageTranslation: translateKey,
  meta: param || {}
});
function isFormGroup(toBeDetermined) {
  return !!toBeDetermined.children;
}
/**
 * @param stringifiedFunction
 * @returns {Function}
 * @description functionFromString returns a function from a stringified function
 * @example functionFromString('/Function((values) => {return values.medicalConditions.toString() === 'true'})/')
 * => (values) => values.medicalConditions.toString() === 'true'
 */
const functionFromString = stringifiedFunction => {
  try {
    const substr = stringifiedFunction.substring(10, stringifiedFunction.length - 2);
    return (0, eval)('(' + substr + ')');
  } catch (e) {
    console.log('Error parsing custom validation function in functionFromString. This issue wont cause an error, current validation was skipped', e);
    return () => false;
  }
};
/**
 * @param validationError - yup validation error
 * @param initialErrors - initial errors object to be merged with the yup validation errors
 * @returns {IValidationErrors} - returns an object with the errors for each field in the form
 * @description formatYupValidationErrors function returns an object with the errors for each field in the form
 * based on the yup validation error formatting to be used in the GenericForm component
 * @example formatYupValidationErrors(yupValidationError, { email: { property: 'email', constraints: { required: { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: {} } } } })
 */
const formatYupValidationErrors = (validationError, initialErrors = {}) => {
  const errors = _extends({}, initialErrors);
  for (const propertyValidationError of validationError.inner) {
    var _propertyValidationEr;
    let errorType = propertyValidationError.type;
    if (!errorType && (_propertyValidationEr = propertyValidationError.params) != null && _propertyValidationEr.regex) {
      var _propertyValidationEr2;
      if (!((_propertyValidationEr2 = propertyValidationError.params) != null && _propertyValidationEr2.regex)) {
        continue;
      }
      errorType = 'pattern';
    }
    if (!get(errors, `${propertyValidationError.path}`)) {
      errors[propertyValidationError.path] = {
        property: propertyValidationError.path,
        constraints: {}
      };
    }
    errors[propertyValidationError.path].constraints[errorType] = _extends({}, getFieldError(errorType, get(propertyValidationError, 'path', null), undefined, get(propertyValidationError.params, `${propertyValidationError.type}`)), {
      fallbackLanguageTranslation: propertyValidationError.message
    });
  }
  return errors;
};
/**
 * @param children
 * @param type - field type
 * @param acc - accumulator, used for recursion
 * @returns {string[]} - array of field names
 * @description getFieldNamesByType function returns an array of field names based on the field type
 * @example getFieldNamesByType(form, EFieldTypes.TEXT) => ['firstName', 'lastName']
 */
const getFieldNamesByType = (children, type, acc = []) => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return getFieldNamesByType(child.children, type, acc);
    }
    if (child.type === type) {
      acc.push(child.fieldName);
    }
    return acc;
  }, acc);
};
const getAllFieldNames = (children, acc = []) => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return getAllFieldNames(child.children, acc);
    }
    acc.push(child.fieldName);
    return acc;
  }, acc);
};
const flattenValues = obj => {
  const flattenedValues = cloneDeep(obj);
  const keys = Object.keys(obj);
  keys.forEach(k => {
    if (typeof obj[k] === 'object' && Object.prototype.propertyIsEnumerable.call(obj[k], 'value')) {
      flattenedValues[k] = obj[k].value;
    }
  });
  return flattenedValues;
};
const valuesToDBFormat = obj => {
  const dbValues = cloneDeep(obj);
  const keys = Object.keys(obj);
  keys.forEach(k => {
    const dbObject = {};
    Object.defineProperty(dbObject, 'value', {
      enumerable: true,
      value: obj[k]
    });
    dbValues[k] = dbObject;
  });
  return dbValues;
};
/**
 * @param obj - object to trim
 * @returns {any} - object with trimmed values
 * @description trimValuesInObject function returns an object with trimmed values
 */
const trimValuesInObject = obj => {
  const trimmedValues = _extends({}, obj);
  const keys = Object.keys(obj);
  keys.forEach(k => obj[k] = typeof obj[k] == 'string' ? trimmedValues[k] = String(obj[k]).trim() : trimmedValues[k] = obj[k]);
  return trimmedValues;
};
const parseForm = form => {
  if (!(form != null && form.formatVersion)) {
    throw new Error('formatVersion is required');
  }
  switch (form.formatVersion) {
    case 1:
      return parseFormV1(form);
    case 2:
      return parseFormV2(form);
    default:
      throw new Error('The parsing function for this version of forms is not implemented');
  }
};

function IsInValidProperties(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'IsInValidProperties',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          return validProperties.includes(value);
        },
        defaultMessage(args) {
          return `${args.property} must be included in ${validProperties.join(', ')}.`;
        }
      }
    });
  };
}

function IsCrossFieldValidations(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'isArrayOfType',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          for (const key in value) {
            if (!Array.isArray(value[key])) {
              return false;
            }
            for (const item of args.value[key]) {
              if (typeof item.errorKey !== 'string' || typeof item.testCondition !== 'string' || item.meta !== undefined && typeof item.meta.param !== 'number') {
                return false;
              }
            }
          }
          return true;
        },
        defaultMessage(args) {
          const invalidIndexes = [];
          const invalidInstances = [];
          for (const key in args.value) {
            if (!Array.isArray(args.value[key])) {
              invalidIndexes.push(key);
            } else {
              for (const item of args.value[key]) {
                if (typeof item.errorKey !== 'string' || typeof item.testCondition !== 'string' || item.meta !== undefined && typeof item.meta.param !== 'number') {
                  invalidInstances.push(key);
                }
              }
            }
          }
          let message = `${args.property} must be an array`;
          if (invalidIndexes.length > 0) {
            const indexesString = invalidIndexes.join(', ');
            message += `.${indexesString}`;
          }
          if (invalidInstances.length > 0) {
            const instancesString = invalidInstances.join(', ');
            message += ` and ${instancesString} must be instances of CrossFieldValidationRule`;
          }
          return message;
        }
      }
    });
  };
}

function isJsonSchemaPropertyValid(property) {
  if (property.id === undefined || typeof property.id !== 'number') {
    return false;
  }
  if (property.maxLength !== undefined && typeof property.maxLength !== 'number') {
    return false;
  }
  if (property.minLength !== undefined && typeof property.minLength !== 'number') {
    return false;
  }
  if (property.pattern !== undefined && typeof property.pattern !== 'string') {
    return false;
  }
  if (property.items !== undefined) {
    const items = property.items;
    if (typeof items !== 'object' || items.type !== 'string' || items.format !== 'date' || typeof items.minLength !== 'number' || typeof items.required !== 'boolean') {
      return false;
    }
  }
  return true;
}
function IsJsonSchemaProperty(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'isJsonSchemaProperty',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          if (typeof value !== 'object') {
            return false;
          }
          for (const key in value) {
            const property = value[key];
            if (property !== null && typeof property === 'object') {
              if (!isJsonSchemaPropertyValid(property)) {
                return false;
              }
            }
          }
          return true;
        },
        defaultMessage(args) {
          return `${args.property} must be a valid IJsonSchemaProperty object.`;
        }
      }
    });
  };
}

function isCrossFieldConditionsParamsValid(params) {
  if (params.required && typeof params.required !== 'string') {
    return false;
  }
  if (params.disabled && typeof params.disabled !== 'string') {
    return false;
  }
  if (params.renderCondition && typeof params.renderCondition !== 'string') {
    return false;
  }
  if (params.disabledCondition && typeof params.disabledCondition !== 'string') {
    return false;
  }
  return true;
}
function IsCrossFieldConditions(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'isCrossFieldConditions',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          if (typeof value !== 'object') {
            return false;
          }
          for (const key in value) {
            if (!isCrossFieldConditionsParamsValid(value[key])) {
              return false;
            }
          }
          return true;
        },
        defaultMessage(args) {
          return `${args.property} must be a valid ICrossFieldConditions object.`;
        }
      }
    });
  };
}

function isFormOverridesValid(override) {
  if (override.fieldTitle !== undefined && typeof override.fieldTitle !== 'string') {
    return false;
  }
  if (override.fieldSubtitle !== undefined && override.fieldSubtitle !== null && typeof override.fieldSubtitle !== 'string') {
    return false;
  }
  if (override.options !== undefined && !Array.isArray(override.options)) {
    return false;
  }
  if (override.placeholder !== undefined && typeof override.placeholder !== 'string') {
    return false;
  }
  if (override.helperText !== undefined && typeof override.helperText !== 'string') {
    return false;
  }
  return true;
}
function IsFormOverrides(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'isFormOverrides',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          if (typeof value !== 'object') {
            return false;
          }
          for (const key in value) {
            const override = value[key];
            if (override !== null && typeof override === 'object') {
              if (!isFormOverridesValid(override)) {
                return false;
              }
            }
          }
          return true;
        },
        defaultMessage(args) {
          return `${args.property} must be a valid IFormOverrides object.`;
        }
      }
    });
  };
}

function IsFormLayout(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'isFormLayout',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          if (typeof value !== 'object') {
            return false;
          }
          for (const key in value) {
            const breakpoints = value[key];
            if (breakpoints !== null && typeof breakpoints === 'object') {
              if (!isRegularBreakpointsValid(breakpoints)) {
                return false;
              }
            }
          }
          return true;
        },
        defaultMessage(args) {
          return `${args.property} must be a valid IFormLayout object.`;
        }
      }
    });
  };
}
function isRegularBreakpointsValid(breakpoints) {
  const validGridSizes = [1, 2, 3, 4, 6, 8, 10, 12, 'auto', undefined];
  for (const key in breakpoints) {
    if (!['lg', 'md', 'sm', 'xl', 'xs'].includes(key)) {
      return false;
    }
    const gridSize = breakpoints[key];
    if (!validGridSizes.includes(gridSize)) {
      return false;
    }
  }
  return true;
}

let IsEqualToConstraint = class IsEqualToConstraint {
  validate(value, args) {
    const [comparisonValue] = args.constraints;
    return value === comparisonValue;
  }
  defaultMessage(args) {
    return `${args.property} must be equal to ${args.constraints[0]}`;
  }
};
IsEqualToConstraint = __decorate([ValidatorConstraint({
  name: 'isEqualTo',
  async: false
})], IsEqualToConstraint);
function IsEqualTo(comparisonValue, validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      constraints: [comparisonValue],
      validator: IsEqualToConstraint
    });
  };
}

function IsKeysValid(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'IsKeysValid',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          if (typeof value !== 'object') {
            return false;
          }
          for (const key in value) {
            if (!validProperties.includes(key)) {
              return false;
            }
          }
          return true;
        },
        defaultMessage(args) {
          return `Keys of ${args.property} must be in [${validProperties.join(', ')}]`;
        }
      }
    });
  };
}

function IsFieldNameValid(validationOptions) {
  return function (object, propertyName) {
    registerDecorator({
      name: 'IsFieldNameValid',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value, args) {
          if (!validProperties.includes(args.value)) {
            return false;
          }
          return true;
        },
        defaultMessage(args) {
          return `Value of ${args.property} must be in [${validProperties.join(', ')}]`;
        }
      }
    });
  };
}

let validProperties = [];
class Option {
  constructor() {
    this.id = void 0;
    this.label = void 0;
    this.value = void 0;
  }
}
__decorate([IsNumber()], Option.prototype, "id", void 0);
__decorate([IsString()], Option.prototype, "label", void 0);
class JsonSchema {
  constructor() {
    this.$schema = void 0;
    this.title = void 0;
    this.description = void 0;
    this.type = void 0;
    this.properties = void 0;
    this.required = void 0;
  }
}
__decorate([IsString(), IsNotEmpty()], JsonSchema.prototype, "$schema", void 0);
__decorate([IsString(), IsNotEmpty()], JsonSchema.prototype, "title", void 0);
__decorate([IsString(), IsNotEmpty()], JsonSchema.prototype, "description", void 0);
__decorate([IsString(), IsNotEmpty()], JsonSchema.prototype, "type", void 0);
__decorate([IsJsonSchemaProperty()], JsonSchema.prototype, "properties", void 0);
__decorate([IsArray(), IsInValidProperties({
  each: true
})], JsonSchema.prototype, "required", void 0);
class CrossFieldValidationRule {
  constructor(obj) {
    this.errorKey = void 0;
    this.testCondition = void 0;
    this.meta = void 0;
    this.errorKey = obj.errorKey;
    this.meta = obj.meta;
    this.testCondition = obj.testCondition;
  }
}
__decorate([IsString({
  message: 'errorKey must be a string.'
}), IsNotEmpty({
  message: 'errorKey is required.'
})], CrossFieldValidationRule.prototype, "errorKey", void 0);
__decorate([IsString({
  message: 'testCondition must be a string.'
}), IsNotEmpty({
  message: 'testCondition is required.'
})], CrossFieldValidationRule.prototype, "testCondition", void 0);
__decorate([IsOptional(), IsObject({
  message: 'meta must be an object.'
}), ValidateNested()], CrossFieldValidationRule.prototype, "meta", void 0);
class CommonChildrenFull {
  constructor() {
    this.type = EFieldTypes.EMPTY;
    this.fieldName = '';
    this.initialValue = '';
    this.templateId = '';
    this.fieldTitle = void 0;
    this.fieldSubtitle = void 0;
    this.placeholder = void 0;
  }
}
__decorate([IsEnum(EFieldTypes)], CommonChildrenFull.prototype, "type", void 0);
__decorate([IsString(), Matches(/^[a-zA-Z$_][a-zA-Z0-9$_]*$/, {
  message: 'fieldName must start with a letter, dollar sign, or underscore, and may only contain letters, digits, dollar signs, and underscores.'
}), MaxLength(30), IsFieldNameValid()], CommonChildrenFull.prototype, "fieldName", void 0);
__decorate([IsString()], CommonChildrenFull.prototype, "templateId", void 0);
__decorate([IsOptional(), IsString()], CommonChildrenFull.prototype, "fieldTitle", void 0);
__decorate([IsOptional(), IsString()], CommonChildrenFull.prototype, "fieldSubtitle", void 0);
__decorate([IsOptional(), IsString()], CommonChildrenFull.prototype, "placeholder", void 0);
class ChildrenTextFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.multiline = void 0;
    this.maxLength = void 0;
    this.type = EFieldTypes.TEXT;
    this.initialValue = null;
    this.disabled = void 0;
  }
}
__decorate([IsOptional(), IsBoolean()], ChildrenTextFull.prototype, "multiline", void 0);
__decorate([IsOptional(), IsString()], ChildrenTextFull.prototype, "maxLength", void 0);
__decorate([IsEqualTo(EFieldTypes.TEXT)], ChildrenTextFull.prototype, "type", void 0);
__decorate([IsEqualTo(null)], ChildrenTextFull.prototype, "initialValue", void 0);
class ChildrenAutoCompleteFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.backendOptions = void 0;
    this.backendSelectType = void 0;
    this.options = void 0;
    this.disabledOptionIds = void 0;
    this.type = EFieldTypes.AUTOCOMPLETE;
    this.initialValue = null;
    this.loading = void 0;
    this.disabled = void 0;
  }
}
__decorate([IsOptional(), IsBoolean()], ChildrenAutoCompleteFull.prototype, "backendOptions", void 0);
__decorate([IsEnum(ESelectTypes), IsOptional()], ChildrenAutoCompleteFull.prototype, "backendSelectType", void 0);
__decorate([IsArray(), ValidateNested({
  each: true
}), Type(() => Option)], ChildrenAutoCompleteFull.prototype, "options", void 0);
__decorate([IsOptional(), IsArray()], ChildrenAutoCompleteFull.prototype, "disabledOptionIds", void 0);
__decorate([IsEqualTo(EFieldTypes.AUTOCOMPLETE)], ChildrenAutoCompleteFull.prototype, "type", void 0);
__decorate([IsEqualTo(null)], ChildrenAutoCompleteFull.prototype, "initialValue", void 0);
__decorate([IsBoolean(), IsOptional()], ChildrenAutoCompleteFull.prototype, "loading", void 0);
__decorate([IsBoolean(), IsOptional()], ChildrenAutoCompleteFull.prototype, "disabled", void 0);
class ChildrenMobilePhoneFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.MOBILE_PHONE;
    this.initialValue = '';
    this.maxLength = void 0;
  }
}
__decorate([IsEqualTo(EFieldTypes.MOBILE_PHONE)], ChildrenMobilePhoneFull.prototype, "type", void 0);
__decorate([IsString()], ChildrenMobilePhoneFull.prototype, "initialValue", void 0);
__decorate([IsString(), IsOptional()], ChildrenMobilePhoneFull.prototype, "maxLength", void 0);
class ChildrenRadioButtonFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.RADIOBUTTON;
    this.initialValue = null;
    this.options = void 0;
  }
}
__decorate([IsEqualTo(EFieldTypes.RADIOBUTTON)], ChildrenRadioButtonFull.prototype, "type", void 0);
__decorate([IsEqualTo(null)], ChildrenRadioButtonFull.prototype, "initialValue", void 0);
__decorate([IsArray(), ValidateNested({
  each: true
}), Type(() => Option)], ChildrenRadioButtonFull.prototype, "options", void 0);
class ChildrenCheckboxFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.CHECKBOX;
    this.placeholder = '';
    this.initialValue = false;
  }
}
__decorate([IsEqualTo(EFieldTypes.CHECKBOX)], ChildrenCheckboxFull.prototype, "type", void 0);
__decorate([IsString()], ChildrenCheckboxFull.prototype, "placeholder", void 0);
__decorate([IsBoolean(), IsEqualTo(false)], ChildrenCheckboxFull.prototype, "initialValue", void 0);
class ChildrenNumberFul extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.NUMBER;
    this.initialValue = 0;
  }
}
__decorate([IsEqualTo(EFieldTypes.NUMBER)], ChildrenNumberFul.prototype, "type", void 0);
__decorate([IsNumber()], ChildrenNumberFul.prototype, "initialValue", void 0);
class ChildrenDateFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.disableFuture = void 0;
    this.disablePast = void 0;
    this.minDate = void 0;
    this.helperText = '';
    this.type = EFieldTypes.DATE;
    this.initialValue = null;
  }
}
__decorate([IsOptional(), IsBoolean()], ChildrenDateFull.prototype, "disableFuture", void 0);
__decorate([IsOptional(), IsBoolean()], ChildrenDateFull.prototype, "disablePast", void 0);
__decorate([IsOptional(), IsDateString()], ChildrenDateFull.prototype, "minDate", void 0);
__decorate([IsNotEmpty()], ChildrenDateFull.prototype, "helperText", void 0);
__decorate([IsEqualTo(EFieldTypes.DATE)], ChildrenDateFull.prototype, "type", void 0);
class ChildrenDateRangeFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.DATE_RANGE;
    this.initialValue = [null, null];
    this.disableFuture = void 0;
    this.disablePast = void 0;
    this.minDate = void 0;
    this.helperText = '';
  }
}
__decorate([IsEqualTo(EFieldTypes.DATE_RANGE)], ChildrenDateRangeFull.prototype, "type", void 0);
__decorate([IsOptional(), IsBoolean()], ChildrenDateRangeFull.prototype, "disableFuture", void 0);
__decorate([IsOptional(), IsBoolean()], ChildrenDateRangeFull.prototype, "disablePast", void 0);
__decorate([IsOptional(), IsISO8601()], ChildrenDateRangeFull.prototype, "minDate", void 0);
__decorate([IsString()], ChildrenDateRangeFull.prototype, "helperText", void 0);
class ChildrenFileFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.FILE;
  }
}
__decorate([IsEqualTo(EFieldTypes.FILE)], ChildrenFileFull.prototype, "type", void 0);
class ChildrenMultipleAutocompleteFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.MULTIPLE_AUTOCOMPLETE;
  }
}
__decorate([IsEqualTo(EFieldTypes.MULTIPLE_AUTOCOMPLETE)], ChildrenMultipleAutocompleteFull.prototype, "type", void 0);
class ChildrenArrayFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.ARRAY;
  }
}
__decorate([IsEqualTo(EFieldTypes.ARRAY)], ChildrenArrayFull.prototype, "type", void 0);
class RegularBreakpoints {
  constructor() {
    this.lg = void 0;
    this.md = void 0;
    this.sm = void 0;
    this.xl = void 0;
    this.xs = void 0;
  }
}
__decorate([IsOptional(), IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])], RegularBreakpoints.prototype, "lg", void 0);
__decorate([IsOptional(), IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])], RegularBreakpoints.prototype, "md", void 0);
__decorate([IsOptional(), IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])], RegularBreakpoints.prototype, "sm", void 0);
__decorate([IsOptional(), IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])], RegularBreakpoints.prototype, "xl", void 0);
__decorate([IsOptional(), IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])], RegularBreakpoints.prototype, "xs", void 0);
class RadioButtonMatrixOptions {
  constructor() {
    this.rows = void 0;
    this.columns = void 0;
    this.grid = void 0;
  }
}
__decorate([IsArray(), ValidateNested({
  each: true
}), Type(() => Option)], RadioButtonMatrixOptions.prototype, "rows", void 0);
__decorate([IsArray(), ValidateNested({
  each: true
}), Type(() => Option)], RadioButtonMatrixOptions.prototype, "columns", void 0);
__decorate([Type(() => RegularBreakpoints)], RadioButtonMatrixOptions.prototype, "grid", void 0);
class ChildrenRadioButtonMatrixFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.RADIOBUTTON_MATRIX;
    this.radioButtonMatrixOptions = void 0;
  }
}
__decorate([IsEqualTo(EFieldTypes.RADIOBUTTON_MATRIX)], ChildrenRadioButtonMatrixFull.prototype, "type", void 0);
__decorate([Type(() => RadioButtonMatrixOptions)], ChildrenRadioButtonMatrixFull.prototype, "radioButtonMatrixOptions", void 0);
class ChildrenEmptyFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.EMPTY;
  }
}
__decorate([IsEqualTo(EFieldTypes.EMPTY)], ChildrenEmptyFull.prototype, "type", void 0);
class ChildrenSwitchFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.SWITCH;
  }
}
__decorate([IsEqualTo(EFieldTypes.SWITCH)], ChildrenSwitchFull.prototype, "type", void 0);
class ChildrenYesNoRadioButtonFull extends CommonChildrenFull {
  constructor(...args) {
    super(...args);
    this.type = EFieldTypes.YES_NO_RADIOBUTTON;
    this.options = void 0;
    this.initialValue = null;
  }
}
__decorate([IsEqualTo(EFieldTypes.YES_NO_RADIOBUTTON)], ChildrenYesNoRadioButtonFull.prototype, "type", void 0);
__decorate([IsArray(), ValidateNested({
  each: true
}), Type(() => Option)], ChildrenYesNoRadioButtonFull.prototype, "options", void 0);
__decorate([IsEqualTo(null)], ChildrenYesNoRadioButtonFull.prototype, "initialValue", void 0);
class FormGroupFull {
  constructor() {
    this.title = void 0;
    this.subtitle = void 0;
    this.children = void 0;
  }
}
__decorate([IsString(), IsOptional()], FormGroupFull.prototype, "title", void 0);
__decorate([IsString(), IsOptional()], FormGroupFull.prototype, "subtitle", void 0);
__decorate([IsArray(), ValidateNested({
  each: true
}), Type(prop => {
  if (prop) {
    var _prop$object, _prop$object2, _prop$object3, _prop$object4, _prop$object5, _prop$object6, _prop$object7, _prop$object8, _prop$object9, _prop$object10, _prop$object11, _prop$object12, _prop$object13, _prop$object14, _prop$object15;
    if (((_prop$object = prop.object) == null ? void 0 : _prop$object.type) === EFieldTypes.TEXT) {
      return ChildrenTextFull;
    }
    if (((_prop$object2 = prop.object) == null ? void 0 : _prop$object2.type) === EFieldTypes.AUTOCOMPLETE) {
      return ChildrenAutoCompleteFull;
    }
    if (((_prop$object3 = prop.object) == null ? void 0 : _prop$object3.type) === EFieldTypes.DATE) {
      return ChildrenDateFull;
    }
    if (((_prop$object4 = prop.object) == null ? void 0 : _prop$object4.type) === EFieldTypes.RADIOBUTTON) {
      return ChildrenRadioButtonFull;
    }
    if (((_prop$object5 = prop.object) == null ? void 0 : _prop$object5.type) === EFieldTypes.CHECKBOX) {
      return ChildrenCheckboxFull;
    }
    if (((_prop$object6 = prop.object) == null ? void 0 : _prop$object6.type) === EFieldTypes.NUMBER) {
      return ChildrenNumberFul;
    }
    if (((_prop$object7 = prop.object) == null ? void 0 : _prop$object7.type) === EFieldTypes.MOBILE_PHONE) {
      return ChildrenMobilePhoneFull;
    }
    if (((_prop$object8 = prop.object) == null ? void 0 : _prop$object8.type) === EFieldTypes.DATE_RANGE) {
      return ChildrenDateRangeFull;
    }
    if (((_prop$object9 = prop.object) == null ? void 0 : _prop$object9.type) === EFieldTypes.FILE) {
      return ChildrenFileFull;
    }
    if (((_prop$object10 = prop.object) == null ? void 0 : _prop$object10.type) === EFieldTypes.MULTIPLE_AUTOCOMPLETE) {
      return ChildrenMultipleAutocompleteFull;
    }
    if (((_prop$object11 = prop.object) == null ? void 0 : _prop$object11.type) === EFieldTypes.ARRAY) {
      return ChildrenArrayFull;
    }
    if (((_prop$object12 = prop.object) == null ? void 0 : _prop$object12.type) === EFieldTypes.RADIOBUTTON_MATRIX) {
      return ChildrenRadioButtonMatrixFull;
    }
    if (((_prop$object13 = prop.object) == null ? void 0 : _prop$object13.type) === EFieldTypes.EMPTY) {
      return ChildrenEmptyFull;
    }
    if (((_prop$object14 = prop.object) == null ? void 0 : _prop$object14.type) === EFieldTypes.SWITCH) {
      return ChildrenSwitchFull;
    }
    if (((_prop$object15 = prop.object) == null ? void 0 : _prop$object15.type) === EFieldTypes.YES_NO_RADIOBUTTON) {
      return ChildrenYesNoRadioButtonFull;
    }
  }
  return Object;
}, {
  discriminator: {
    property: 'type',
    subTypes: [{
      value: ChildrenTextFull,
      name: EFieldTypes.TEXT
    }, {
      value: ChildrenAutoCompleteFull,
      name: EFieldTypes.AUTOCOMPLETE
    }, {
      value: ChildrenDateFull,
      name: EFieldTypes.DATE
    }, {
      value: ChildrenRadioButtonFull,
      name: EFieldTypes.RADIOBUTTON
    }, {
      value: ChildrenCheckboxFull,
      name: EFieldTypes.CHECKBOX
    }, {
      value: ChildrenNumberFul,
      name: EFieldTypes.NUMBER
    }, {
      value: ChildrenMobilePhoneFull,
      name: EFieldTypes.MOBILE_PHONE
    }, {
      value: ChildrenDateRangeFull,
      name: EFieldTypes.DATE_RANGE
    }, {
      value: ChildrenFileFull,
      name: EFieldTypes.FILE
    }, {
      value: ChildrenMultipleAutocompleteFull,
      name: EFieldTypes.MULTIPLE_AUTOCOMPLETE
    }, {
      value: ChildrenArrayFull,
      name: EFieldTypes.ARRAY
    }, {
      value: ChildrenRadioButtonMatrixFull,
      name: EFieldTypes.RADIOBUTTON_MATRIX
    }, {
      value: ChildrenEmptyFull,
      name: EFieldTypes.EMPTY
    }, {
      value: ChildrenSwitchFull,
      name: EFieldTypes.SWITCH
    }, {
      value: ChildrenYesNoRadioButtonFull,
      name: EFieldTypes.YES_NO_RADIOBUTTON
    }]
  }
})], FormGroupFull.prototype, "children", void 0);
class FormV2Full {
  constructor() {
    this.formatVersion = void 0;
    this.validationObject = void 0;
    this.crossFieldValidations = void 0;
    this.crossFieldConditions = void 0;
    this.overrides = void 0;
    this.formLayout = void 0;
    this.children = void 0;
    this.submitButtonText = void 0;
    this.cancelButtonText = void 0;
  }
}
__decorate([IsNumber()], FormV2Full.prototype, "formatVersion", void 0);
__decorate([Type(() => JsonSchema), ValidateNested()], FormV2Full.prototype, "validationObject", void 0);
__decorate([IsCrossFieldValidations(), IsKeysValid()], FormV2Full.prototype, "crossFieldValidations", void 0);
__decorate([IsCrossFieldConditions(), IsKeysValid()], FormV2Full.prototype, "crossFieldConditions", void 0);
__decorate([IsFormOverrides(), IsKeysValid()], FormV2Full.prototype, "overrides", void 0);
__decorate([IsFormLayout(), IsKeysValid()], FormV2Full.prototype, "formLayout", void 0);
__decorate([IsArray(), ValidateNested({
  each: true
}), Type(prop => {
  if (prop) {
    var _prop$object16, _prop$object17, _prop$object18, _prop$object19, _prop$object20, _prop$object21, _prop$object22, _prop$object23, _prop$object24, _prop$object25, _prop$object26, _prop$object27, _prop$object28, _prop$object29, _prop$object30, _prop$object31;
    if ((_prop$object16 = prop.object) != null && _prop$object16.children) {
      return FormGroupFull;
    }
    if (((_prop$object17 = prop.object) == null ? void 0 : _prop$object17.type) === EFieldTypes.TEXT) {
      return ChildrenTextFull;
    }
    if (((_prop$object18 = prop.object) == null ? void 0 : _prop$object18.type) === EFieldTypes.AUTOCOMPLETE) {
      return ChildrenAutoCompleteFull;
    }
    if (((_prop$object19 = prop.object) == null ? void 0 : _prop$object19.type) === EFieldTypes.DATE) {
      return ChildrenDateFull;
    }
    if (((_prop$object20 = prop.object) == null ? void 0 : _prop$object20.type) === EFieldTypes.RADIOBUTTON) {
      return ChildrenRadioButtonFull;
    }
    if (((_prop$object21 = prop.object) == null ? void 0 : _prop$object21.type) === EFieldTypes.CHECKBOX) {
      return ChildrenCheckboxFull;
    }
    if (((_prop$object22 = prop.object) == null ? void 0 : _prop$object22.type) === EFieldTypes.NUMBER) {
      return ChildrenNumberFul;
    }
    if (((_prop$object23 = prop.object) == null ? void 0 : _prop$object23.type) === EFieldTypes.MOBILE_PHONE) {
      return ChildrenMobilePhoneFull;
    }
    if (((_prop$object24 = prop.object) == null ? void 0 : _prop$object24.type) === EFieldTypes.DATE_RANGE) {
      return ChildrenDateRangeFull;
    }
    if (((_prop$object25 = prop.object) == null ? void 0 : _prop$object25.type) === EFieldTypes.FILE) {
      return ChildrenFileFull;
    }
    if (((_prop$object26 = prop.object) == null ? void 0 : _prop$object26.type) === EFieldTypes.MULTIPLE_AUTOCOMPLETE) {
      return ChildrenMultipleAutocompleteFull;
    }
    if (((_prop$object27 = prop.object) == null ? void 0 : _prop$object27.type) === EFieldTypes.ARRAY) {
      return ChildrenArrayFull;
    }
    if (((_prop$object28 = prop.object) == null ? void 0 : _prop$object28.type) === EFieldTypes.RADIOBUTTON_MATRIX) {
      return ChildrenRadioButtonMatrixFull;
    }
    if (((_prop$object29 = prop.object) == null ? void 0 : _prop$object29.type) === EFieldTypes.EMPTY) {
      return ChildrenEmptyFull;
    }
    if (((_prop$object30 = prop.object) == null ? void 0 : _prop$object30.type) === EFieldTypes.SWITCH) {
      return ChildrenSwitchFull;
    }
    if (((_prop$object31 = prop.object) == null ? void 0 : _prop$object31.type) === EFieldTypes.YES_NO_RADIOBUTTON) {
      return ChildrenYesNoRadioButtonFull;
    }
  }
  return Object;
}, {
  discriminator: {
    property: 'type',
    subTypes: [{
      value: FormGroupFull,
      name: 'group'
    }, {
      value: ChildrenTextFull,
      name: EFieldTypes.TEXT
    }, {
      value: ChildrenAutoCompleteFull,
      name: EFieldTypes.AUTOCOMPLETE
    }, {
      value: ChildrenDateFull,
      name: EFieldTypes.DATE
    }, {
      value: ChildrenRadioButtonFull,
      name: EFieldTypes.RADIOBUTTON
    }, {
      value: ChildrenCheckboxFull,
      name: EFieldTypes.CHECKBOX
    }, {
      value: ChildrenNumberFul,
      name: EFieldTypes.NUMBER
    }, {
      value: ChildrenMobilePhoneFull,
      name: EFieldTypes.MOBILE_PHONE
    }, {
      value: ChildrenDateRangeFull,
      name: EFieldTypes.DATE_RANGE
    }, {
      value: ChildrenFileFull,
      name: EFieldTypes.FILE
    }, {
      value: ChildrenMultipleAutocompleteFull,
      name: EFieldTypes.MULTIPLE_AUTOCOMPLETE
    }, {
      value: ChildrenArrayFull,
      name: EFieldTypes.ARRAY
    }, {
      value: ChildrenRadioButtonMatrixFull,
      name: EFieldTypes.RADIOBUTTON_MATRIX
    }, {
      value: ChildrenEmptyFull,
      name: EFieldTypes.EMPTY
    }, {
      value: ChildrenSwitchFull,
      name: EFieldTypes.SWITCH
    }, {
      value: ChildrenYesNoRadioButtonFull,
      name: EFieldTypes.YES_NO_RADIOBUTTON
    }]
  }
})], FormV2Full.prototype, "children", void 0);
__decorate([IsString()], FormV2Full.prototype, "submitButtonText", void 0);
__decorate([IsOptional(), IsString()], FormV2Full.prototype, "cancelButtonText", void 0);
class Validate {
  constructor() {
    this.form = void 0;
  }
}
__decorate([Type(() => FormV2Full), ValidateNested()], Validate.prototype, "form", void 0);
const validateSchema = async form => {
  var _form$validationObjec;
  validProperties = Object.keys(form == null || (_form$validationObjec = form.validationObject) == null ? void 0 : _form$validationObjec.properties) || [];
  const validateInstance = plainToClass(Validate, {
    form: cloneDeep(form)
  });
  return await validate(validateInstance);
};

export { EFieldTypes, ESelectTypes, FormV2Full, buildYupSchema, createErrorObject, flattenValues, formatYupValidationErrors, functionFromString, getAllFieldNames, getErrorMessages, getFieldError, getInitialValues, getRuleValueFromSchema, isFormGroup, makeCrossFieldValidation, parseForm, parseToFormV2, validateSchema, validateValues, valuesToDBFormat };
//# sourceMappingURL=index.modern.mjs.map
