import 'reflect-metadata';
import {
  IsNumber,
  IsString,
  IsOptional,
  IsArray,
  ValidateNested,
  validate,
  IsNotEmpty,
  IsObject,
  IsBoolean,
  IsEnum,
  Matches,
  IsDateString,
  ValidationError,
  IsISO8601,
  IsIn,
  MaxLength,
} from 'class-validator';
import { Type, plainToClass } from 'class-transformer';
import {
  EFieldTypes,
  ESelectTypes,
  EnumType,
  GridSize,
  ICrossFieldValidationRule,
  ICrossFieldValidations,
  IJsonSchema,
  IOption,
  IProperty,
  IRegularBreakpoints,
} from './modelsCommon';
import {
  ICrossFieldConditions,
  IFormLayout,
  IFormOverrides,
  IFormV2Full,
} from './models';
import {
  IChildrenArrayFull,
  IChildrenAutoCompleteFull,
  IChildrenCheckboxFull,
  IChildrenDateFull,
  IChildrenDateRangeFull,
  IChildrenEmptyFull,
  IChildrenFileFull,
  IChildrenMobilePhoneFull,
  IChildrenMultipleAutocompleteFull,
  IChildrenNumberFull,
  IChildrenRadioButtonFull,
  IChildrenRadioButtonMatrixFull,
  IChildrenSwitchFull,
  IChildrenTextFull,
  IChildrenYesNoRadioButtonFull,
  ICommonChildrenFull,
  IFormGroupFull,
  IRadioButtonMatrixOptions,
} from './inputsModels';
import { IsInValidProperties } from './decorators/IsInRequiredProperties';
import { IsCrossFieldValidations } from './decorators/IsCrossFieldValidations';
import { IsJsonSchemaProperty } from './decorators/IsJsonSchemaProperty';
import { IsCrossFieldConditions } from './decorators/IsCrossFieldConditions';
import { IsFormOverrides } from './decorators/isFormOverridesValid';
import { IsFormLayout } from './decorators/IsFormLayout';
import { IsEqualTo } from './decorators/IsEqualTo';
import cloneDeep from 'lodash.clonedeep';
import { IsKeysValid } from './decorators/isKeyValid';
import { IsFieldNameValid } from './decorators/isFieldNameValid';

export let validProperties: string[] = [];

export class Option<T extends EnumType> implements IOption<T> {
  @IsNumber()
  id!: number;

  @IsString()
  label!: string;

  value!: T;
}

class JsonSchema<T extends EnumType> implements IJsonSchema<T> {
  @IsString()
  @IsNotEmpty()
  $schema!: string;

  @IsString()
  @IsNotEmpty()
  title!: string;

  @IsString()
  @IsNotEmpty()
  description!: string;

  @IsString()
  @IsNotEmpty()
  type!: string;

  @IsJsonSchemaProperty()
  properties!: IProperty<T>;

  @IsArray()
  @IsInValidProperties({ each: true })
  required!: Array<T>;
}

export class CrossFieldValidationRule implements ICrossFieldValidationRule {
  @IsString({ message: 'errorKey must be a string.' })
  @IsNotEmpty({ message: 'errorKey is required.' })
  errorKey: string;

  @IsString({ message: 'testCondition must be a string.' })
  @IsNotEmpty({ message: 'testCondition is required.' })
  testCondition: string;

  @IsOptional()
  @IsObject({ message: 'meta must be an object.' })
  @ValidateNested()
  meta?: { param: number };

  constructor(obj: ICrossFieldValidationRule) {
    this.errorKey = obj.errorKey;
    this.meta = obj.meta;
    this.testCondition = obj.testCondition;
  }
}

export class CommonChildrenFull implements ICommonChildrenFull<string> {
  @IsEnum(EFieldTypes)
  type: EFieldTypes = EFieldTypes.EMPTY;
  @IsString()
  @Matches(/^[a-zA-Z$_][a-zA-Z0-9$_]*$/, {
    message:
      'fieldName must start with a letter, dollar sign, or underscore, and may only contain letters, digits, dollar signs, and underscores.',
  })
  @MaxLength(30)
  @IsFieldNameValid()
  fieldName = '';
  initialValue: string | null | boolean | number | false | [null, null] = '';
  @IsString()
  templateId = '';
  @IsOptional()
  @IsString()
  fieldTitle?: string;
  @IsOptional()
  @IsString()
  fieldSubtitle?: string;
  @IsOptional()
  @IsString()
  placeholder?: string;
}

export class ChildrenTextFull
  extends CommonChildrenFull
  implements IChildrenTextFull<string>
{
  @IsOptional()
  @IsBoolean()
  multiline?: boolean;
  @IsOptional()
  @IsString()
  maxLength?: string;
  @IsEqualTo(EFieldTypes.TEXT)
  type: EFieldTypes.TEXT = EFieldTypes.TEXT;
  @IsEqualTo(null)
  initialValue = null;
  disabled?: boolean | undefined;
}

export class ChildrenAutoCompleteFull
  extends CommonChildrenFull
  implements IChildrenAutoCompleteFull<string>
{
  @IsOptional()
  @IsBoolean()
  backendOptions?: boolean;

  @IsEnum(ESelectTypes)
  @IsOptional()
  backendSelectType?: ESelectTypes;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Option)
  options!: IOption<any>[];

  @IsOptional()
  @IsArray()
  disabledOptionIds?: number[];

  @IsEqualTo(EFieldTypes.AUTOCOMPLETE)
  type: EFieldTypes.AUTOCOMPLETE = EFieldTypes.AUTOCOMPLETE;

  @IsEqualTo(null)
  initialValue = null;

  @IsBoolean()
  @IsOptional()
  loading?: boolean | undefined;

  @IsBoolean()
  @IsOptional()
  disabled?: boolean | undefined;
}

class ChildrenMobilePhoneFull
  extends CommonChildrenFull
  implements IChildrenMobilePhoneFull<string>
{
  @IsEqualTo(EFieldTypes.MOBILE_PHONE)
  type: EFieldTypes.MOBILE_PHONE = EFieldTypes.MOBILE_PHONE;

  @IsString()
  initialValue = '';

  @IsString()
  @IsOptional()
  maxLength?: string;
}

class ChildrenRadioButtonFull
  extends CommonChildrenFull
  implements IChildrenRadioButtonFull<string>
{
  @IsEqualTo(EFieldTypes.RADIOBUTTON)
  type: EFieldTypes.RADIOBUTTON = EFieldTypes.RADIOBUTTON;

  @IsEqualTo(null)
  initialValue = null;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Option)
  options!: IOption<any>[];
}

class ChildrenCheckboxFull
  extends CommonChildrenFull
  implements IChildrenCheckboxFull<string>
{
  @IsEqualTo(EFieldTypes.CHECKBOX)
  type: EFieldTypes.CHECKBOX = EFieldTypes.CHECKBOX;

  @IsString()
  placeholder = '';

  @IsBoolean()
  @IsEqualTo(false)
  initialValue = false;
}

class ChildrenNumberFul
  extends CommonChildrenFull
  implements IChildrenNumberFull<string>
{
  @IsEqualTo(EFieldTypes.NUMBER)
  type: EFieldTypes.NUMBER = EFieldTypes.NUMBER;

  @IsNumber()
  initialValue = 0;
}

export class ChildrenDateFull
  extends CommonChildrenFull
  implements IChildrenDateFull<string>
{
  @IsOptional()
  @IsBoolean()
  disableFuture?: boolean;

  @IsOptional()
  @IsBoolean()
  disablePast?: boolean;

  @IsOptional()
  @IsDateString()
  minDate?: any;

  @IsNotEmpty()
  helperText = '';

  @IsEqualTo(EFieldTypes.DATE)
  type: EFieldTypes.DATE = EFieldTypes.DATE;
  initialValue = null;
}

class ChildrenDateRangeFull
  extends CommonChildrenFull
  implements IChildrenDateRangeFull<string>
{
  @IsEqualTo(EFieldTypes.DATE_RANGE)
  type: EFieldTypes.DATE_RANGE = EFieldTypes.DATE_RANGE;

  initialValue: [null, null] = [null, null];

  @IsOptional()
  @IsBoolean()
  disableFuture?: boolean;

  @IsOptional()
  @IsBoolean()
  disablePast?: boolean;

  @IsOptional()
  @IsISO8601()
  minDate?: any;

  @IsString()
  helperText = '';
}

class ChildrenFileFull
  extends CommonChildrenFull
  implements IChildrenFileFull<string>
{
  @IsEqualTo(EFieldTypes.FILE)
  type: EFieldTypes.FILE = EFieldTypes.FILE;
}

class ChildrenMultipleAutocompleteFull
  extends CommonChildrenFull
  implements IChildrenMultipleAutocompleteFull<string>
{
  @IsEqualTo(EFieldTypes.MULTIPLE_AUTOCOMPLETE)
  type: EFieldTypes.MULTIPLE_AUTOCOMPLETE = EFieldTypes.MULTIPLE_AUTOCOMPLETE;
}

class ChildrenArrayFull
  extends CommonChildrenFull
  implements IChildrenArrayFull<string>
{
  @IsEqualTo(EFieldTypes.ARRAY)
  type: EFieldTypes.ARRAY = EFieldTypes.ARRAY;
}

class RegularBreakpoints implements IRegularBreakpoints {
  @IsOptional()
  @IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])
  lg?: GridSize | undefined;

  @IsOptional()
  @IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])
  md?: GridSize | undefined;

  @IsOptional()
  @IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])
  sm?: GridSize | undefined;

  @IsOptional()
  @IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])
  xl?: GridSize | undefined;

  @IsOptional()
  @IsIn(['auto', 1, 2, 3, 4, 6, 8, 10, 12])
  xs?: GridSize | undefined;
}

class RadioButtonMatrixOptions implements IRadioButtonMatrixOptions {
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Option)
  rows!: IOption<any>[];

  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Option)
  columns!: IOption<any>[];

  @Type(() => RegularBreakpoints)
  grid?: IRegularBreakpoints;
}

class ChildrenRadioButtonMatrixFull
  extends CommonChildrenFull
  implements IChildrenRadioButtonMatrixFull<string>
{
  @IsEqualTo(EFieldTypes.RADIOBUTTON_MATRIX)
  type: EFieldTypes.RADIOBUTTON_MATRIX = EFieldTypes.RADIOBUTTON_MATRIX;

  @Type(() => RadioButtonMatrixOptions)
  radioButtonMatrixOptions!: RadioButtonMatrixOptions;
}

class ChildrenEmptyFull
  extends CommonChildrenFull
  implements IChildrenEmptyFull<string>
{
  @IsEqualTo(EFieldTypes.EMPTY)
  type: EFieldTypes.EMPTY = EFieldTypes.EMPTY;
}

class ChildrenSwitchFull
  extends CommonChildrenFull
  implements IChildrenSwitchFull<string>
{
  @IsEqualTo(EFieldTypes.SWITCH)
  type: EFieldTypes.SWITCH = EFieldTypes.SWITCH;
}

class ChildrenYesNoRadioButtonFull
  extends CommonChildrenFull
  implements IChildrenYesNoRadioButtonFull<string>
{
  @IsEqualTo(EFieldTypes.YES_NO_RADIOBUTTON)
  type: EFieldTypes.YES_NO_RADIOBUTTON = EFieldTypes.YES_NO_RADIOBUTTON;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Option)
  options!: IOption<boolean>[];

  @IsEqualTo(null)
  initialValue = null;
}

export class FormGroupFull implements IFormGroupFull<string> {
  @IsString()
  @IsOptional()
  title?: string;

  @IsString()
  @IsOptional()
  subtitle?: string;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(
    (prop) => {
      if (prop) {
        if (prop.object?.type === EFieldTypes.TEXT) {
          return ChildrenTextFull;
        }
        if (prop.object?.type === EFieldTypes.AUTOCOMPLETE) {
          return ChildrenAutoCompleteFull;
        }
        if (prop.object?.type === EFieldTypes.DATE) {
          return ChildrenDateFull;
        }
        if (prop.object?.type === EFieldTypes.RADIOBUTTON) {
          return ChildrenRadioButtonFull;
        }
        if (prop.object?.type === EFieldTypes.CHECKBOX) {
          return ChildrenCheckboxFull;
        }
        if (prop.object?.type === EFieldTypes.NUMBER) {
          return ChildrenNumberFul;
        }
        if (prop.object?.type === EFieldTypes.MOBILE_PHONE) {
          return ChildrenMobilePhoneFull;
        }
        if (prop.object?.type === EFieldTypes.DATE_RANGE) {
          return ChildrenDateRangeFull;
        }
        if (prop.object?.type === EFieldTypes.FILE) {
          return ChildrenFileFull;
        }
        if (prop.object?.type === EFieldTypes.MULTIPLE_AUTOCOMPLETE) {
          return ChildrenMultipleAutocompleteFull;
        }
        if (prop.object?.type === EFieldTypes.ARRAY) {
          return ChildrenArrayFull;
        }
        if (prop.object?.type === EFieldTypes.RADIOBUTTON_MATRIX) {
          return ChildrenRadioButtonMatrixFull;
        }
        if (prop.object?.type === EFieldTypes.EMPTY) {
          return ChildrenEmptyFull;
        }
        if (prop.object?.type === EFieldTypes.SWITCH) {
          return ChildrenSwitchFull;
        }
        if (prop.object?.type === EFieldTypes.YES_NO_RADIOBUTTON) {
          return ChildrenYesNoRadioButtonFull;
        }
      }
      return Object;
    },
    {
      discriminator: {
        property: 'type',
        subTypes: [
          { value: ChildrenTextFull, name: EFieldTypes.TEXT },
          { value: ChildrenAutoCompleteFull, name: EFieldTypes.AUTOCOMPLETE },
          { value: ChildrenDateFull, name: EFieldTypes.DATE },
          { value: ChildrenRadioButtonFull, name: EFieldTypes.RADIOBUTTON },
          { value: ChildrenCheckboxFull, name: EFieldTypes.CHECKBOX },
          { value: ChildrenNumberFul, name: EFieldTypes.NUMBER },
          { value: ChildrenMobilePhoneFull, name: EFieldTypes.MOBILE_PHONE },
          { value: ChildrenDateRangeFull, name: EFieldTypes.DATE_RANGE },
          { value: ChildrenFileFull, name: EFieldTypes.FILE },
          {
            value: ChildrenMultipleAutocompleteFull,
            name: EFieldTypes.MULTIPLE_AUTOCOMPLETE,
          },
          { value: ChildrenArrayFull, name: EFieldTypes.ARRAY },
          {
            value: ChildrenRadioButtonMatrixFull,
            name: EFieldTypes.RADIOBUTTON_MATRIX,
          },
          { value: ChildrenEmptyFull, name: EFieldTypes.EMPTY },
          { value: ChildrenSwitchFull, name: EFieldTypes.SWITCH },
          {
            value: ChildrenYesNoRadioButtonFull,
            name: EFieldTypes.YES_NO_RADIOBUTTON,
          },
        ],
      },
    },
  )
  children!: Array<
    | ChildrenTextFull
    | ChildrenAutoCompleteFull
    | ChildrenDateFull
    | ChildrenRadioButtonFull
    | ChildrenCheckboxFull
    | ChildrenNumberFul
    | ChildrenMobilePhoneFull
    | ChildrenDateRangeFull
    | ChildrenFileFull
    | ChildrenMultipleAutocompleteFull
    | ChildrenArrayFull
    | ChildrenRadioButtonMatrixFull
    | ChildrenEmptyFull
    | ChildrenSwitchFull
    | ChildrenYesNoRadioButtonFull
  >;
}

export class FormV2Full implements IFormV2Full<string> {
  @IsNumber()
  formatVersion!: 2;

  @Type(() => JsonSchema)
  @ValidateNested()
  validationObject!: JsonSchema<string>;

  @IsCrossFieldValidations()
  @IsKeysValid()
  crossFieldValidations!: ICrossFieldValidations<string>;

  @IsCrossFieldConditions()
  @IsKeysValid()
  crossFieldConditions!: ICrossFieldConditions<string>;

  @IsFormOverrides()
  @IsKeysValid()
  overrides!: IFormOverrides<string>;

  @IsFormLayout()
  @IsKeysValid()
  formLayout!: IFormLayout<string>;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(
    (prop) => {
      if (prop) {
        if (prop.object?.children) {
          return FormGroupFull;
        }
        if (prop.object?.type === EFieldTypes.TEXT) {
          return ChildrenTextFull;
        }
        if (prop.object?.type === EFieldTypes.AUTOCOMPLETE) {
          return ChildrenAutoCompleteFull;
        }
        if (prop.object?.type === EFieldTypes.DATE) {
          return ChildrenDateFull;
        }
        if (prop.object?.type === EFieldTypes.RADIOBUTTON) {
          return ChildrenRadioButtonFull;
        }
        if (prop.object?.type === EFieldTypes.CHECKBOX) {
          return ChildrenCheckboxFull;
        }
        if (prop.object?.type === EFieldTypes.NUMBER) {
          return ChildrenNumberFul;
        }
        if (prop.object?.type === EFieldTypes.MOBILE_PHONE) {
          return ChildrenMobilePhoneFull;
        }
        if (prop.object?.type === EFieldTypes.DATE_RANGE) {
          return ChildrenDateRangeFull;
        }
        if (prop.object?.type === EFieldTypes.FILE) {
          return ChildrenFileFull;
        }
        if (prop.object?.type === EFieldTypes.MULTIPLE_AUTOCOMPLETE) {
          return ChildrenMultipleAutocompleteFull;
        }
        if (prop.object?.type === EFieldTypes.ARRAY) {
          return ChildrenArrayFull;
        }
        if (prop.object?.type === EFieldTypes.RADIOBUTTON_MATRIX) {
          return ChildrenRadioButtonMatrixFull;
        }
        if (prop.object?.type === EFieldTypes.EMPTY) {
          return ChildrenEmptyFull;
        }
        if (prop.object?.type === EFieldTypes.SWITCH) {
          return ChildrenSwitchFull;
        }
        if (prop.object?.type === EFieldTypes.YES_NO_RADIOBUTTON) {
          return ChildrenYesNoRadioButtonFull;
        }
      }
      return Object;
    },
    {
      discriminator: {
        property: 'type',
        subTypes: [
          { value: FormGroupFull, name: 'group' },
          { value: ChildrenTextFull, name: EFieldTypes.TEXT },
          { value: ChildrenAutoCompleteFull, name: EFieldTypes.AUTOCOMPLETE },
          { value: ChildrenDateFull, name: EFieldTypes.DATE },
          { value: ChildrenRadioButtonFull, name: EFieldTypes.RADIOBUTTON },
          { value: ChildrenCheckboxFull, name: EFieldTypes.CHECKBOX },
          { value: ChildrenNumberFul, name: EFieldTypes.NUMBER },
          { value: ChildrenMobilePhoneFull, name: EFieldTypes.MOBILE_PHONE },
          { value: ChildrenDateRangeFull, name: EFieldTypes.DATE_RANGE },
          { value: ChildrenFileFull, name: EFieldTypes.FILE },
          {
            value: ChildrenMultipleAutocompleteFull,
            name: EFieldTypes.MULTIPLE_AUTOCOMPLETE,
          },
          { value: ChildrenArrayFull, name: EFieldTypes.ARRAY },
          {
            value: ChildrenRadioButtonMatrixFull,
            name: EFieldTypes.RADIOBUTTON_MATRIX,
          },
          { value: ChildrenEmptyFull, name: EFieldTypes.EMPTY },
          { value: ChildrenSwitchFull, name: EFieldTypes.SWITCH },
          {
            value: ChildrenYesNoRadioButtonFull,
            name: EFieldTypes.YES_NO_RADIOBUTTON,
          },
        ],
      },
    },
  )
  children!: Array<
    | FormGroupFull
    | ChildrenTextFull
    | ChildrenAutoCompleteFull
    | ChildrenDateFull
    | ChildrenRadioButtonFull
    | ChildrenCheckboxFull
    | ChildrenNumberFul
    | ChildrenMobilePhoneFull
    | ChildrenDateRangeFull
    | ChildrenFileFull
    | ChildrenMultipleAutocompleteFull
    | ChildrenArrayFull
    | ChildrenRadioButtonMatrixFull
    | ChildrenEmptyFull
    | ChildrenSwitchFull
    | ChildrenYesNoRadioButtonFull
  >;

  @IsString()
  submitButtonText!: string;

  @IsOptional()
  @IsString()
  cancelButtonText?: string;
}

class Validate {
  @Type(() => FormV2Full)
  @ValidateNested()
  form!: FormV2Full;
}

export const validateSchema = async (
  form: FormV2Full,
): Promise<ValidationError[]> => {
  validProperties = Object.keys(form?.validationObject?.properties) || [];

  const validateInstance = plainToClass(Validate, { form: cloneDeep(form) });

  return await validate(validateInstance);
};
